	LIST	P = 16F877, R = DEC
	#include	p16f877.inc

	__CONFIG	B'11111100111010'

	#include	displej.inc	; makro na obsuhu displeje
	#include	parit.inc		; makro pro prijimani a vysilani s paritou
	#include	div16u.inc	; makro s algoritmem deleni
	#include	zobraz16.inc	; makro s algoritmem zobrazeni 16bitoveho cisla
	#include	znaky.inc		; definice zakladnich znaku dipleje

;---------------------------------------------
;||||||||||| KONSTANTY PROGRAMU ||||||||||||||
;---------------------------------------------

#DEFINE	DB4	PORTE,2		; PINY PRO OVLADANI DISPLEJE
#DEFINE	DB5	PORTE,1
#DEFINE	DB6	PORTE,0
#DEFINE	DB7	PORTA,4
#DEFINE	E	PORTB,1
#DEFINE	RW	PORTB,2
#DEFINE	RS	PORTB,3

OFFRAM	EQU	H'20'		; ZACATEK VOLNE PAMETI "RAM"

	CBLOCK	OFFRAM
	TEMP:1			; VYPOCET PARITY
	PARITREG:1

	ZNAK:1			; CITAC VYSILANYCH ZNAKU

	DDATA:1			; POMOCNA PROM. OBSLUHY DISPLEJE
	TMH:1			; POMOCNA PROM. DVOJITEHO "DELAY" CYKLU
	TML:1

	AA:2			; OPERANDY A VYSLEDEK DELENI
	BB:2
	CC:2
	ARIT00:1			; POMOCNA PROM. DELENI

	XXX:1
	ENDC

;----------------------------------------------------------
;||||||||||||||||  HLAVNI PROGRAM |||||||||||||||||||||||||
;----------------------------------------------------------

	ORG	0
	NOP
	NOP
	NOP

	MOVLW	B'01100100'	; NASTAV VYSILAC PRO...
	banksel	TXSTA		; ...ASYNCHRONNI 9BITOVY PRENOST
	MOVWF	TXSTA
	MOVLW	B'11010000'	; NASTAV PRIJIMAC PRO...
	banksel	RCSTA		; ...ASYNCHRONNI 9BITOVY PRIJEM
	MOVWF	RCSTA
	MOVLW	D'129'		; NASTAV KOMUNIKACNI RYCHLOST,...
	banksel	SPBRG		; ...V NASEM PRIPADE 9600 BAUDU
	MOVWF	SPBRG
	banksel	ADCON1
	MOVLW	B'00000110'	; NASTAV PORTA JAKO "LOGICKY"
	MOVWF	ADCON1
	banksel	0

	CALL	INITDISP		; INICIALIZACE DISPLEJE

	MOVLW	D'255'		; (PRISTI HODNOTA JE NULA)
	MOVWF	ZNAK
LOOP	INCF	ZNAK,F		; CITAC VYSILANYCH ZNAKU
	BTFSS	PIR1,TXIF		; CEKANI NA UVOLNENI VYSILACIHO BUFFERU
	GOTO	$ - 1
	MOVF	ZNAK,W		; VYSLANI ZNAKU
	CALL	TX_PARIT
	BTFSS	PIR1,RCIF		; CEKANI NA PRIJETI ZNAKU
	GOTO	$ - 1
	BTFSC	RCSTA,FERR	; DETEKCE CHYBY RAMCE
	GOTO	CHYBA
	BTFSC	RCSTA,OERR	; DETEKCE CHYBY PRETECENI BUFFERU
	GOTO	CHYBA
	MOVLW	D'0'		; NASTAVENI POCATECNI POLOHY DISPLEJE
	CALL	SETPOS
	CALL	RC_PARIT		; PRECTENI ZNAKU
	MOVWF	XXX		; USCHOVANI DO "XXX"
	MOVLW	ZNAK_O		; VYBER ZNAK "O" (OK)
	BTFSS	STATUS,Z		; POKUD BYLA DETEKOVANA CHYBA PARITY,...
	MOVLW	ZNAK_E		; ...POTOM VYBER ZNAK "E" (ERROR)
	CALL	PUTZNAK		; VYPIS TENTO ZNAK
	MOVLW	ZNAK_SP		; - MEZERA
	CALL	PUTZNAK
	MOVLW	ZNAK_0
	BTFSC	STATUS,DC		; - VYPIS PARITNI BIT, STALE USCHOVANY V "DC"
	MOVLW	ZNAK_1
	CALL	PUTZNAK
	MOVLW	ZNAK_SP		; - MEZERA
	CALL	PUTZNAK
	MOVF	XXX,W		; ZOBRAZ (DEKADICKY) KOD...
	CALL	ZOBRAZ16		; ...PRIJATEHO ZNAKU
	MOVLW	D'25'		; VYTVOR "PRIMERENOU" PRODLEVU
	MOVWF	XXX
	MOVLW	D'255'
	CALL	DDELAY
	DECFSZ	XXX,F
	GOTO	$ - 3
	GOTO	LOOP		; OPAKUJ CYKLUS
CHYBA	MOVF	RCREG,W		; ODSTRAN CHYBNY BYTE
	BTFSS	RCSTA,OERR
	GOTO	LOOP		; POKUD NENASTALA CHYBA PRETECENI, VRAT SE
	BCF	RCSTA,CREN	; VYPNI...
	BSF	RCSTA,CREN	; ...A ZAPNI PRIJIMAC (RESET)
	GOTO	LOOP

; DVOJICE PODPROGRAMU PRO SERIOVY PRENOS S PARITOU
; - PARITREG1 A PARITREG2, RESPEKTIVE TEMP1 A TEMP2 BUDOU MIT STEJNE ADRESY
	PARIT_COM	PARITREG,PARITREG,TEMP,TEMP

; SOUBOR PODPROGRAMU NA ZAKLADNI OBSLUHU DISPLEJE	
	DISPLEJ	TMH,TML,DDATA,DB4,DB5,DB6,DB7,E,RW,RS

; 16BITOVE DELENI BEZ ZNAMINKA
DIV16U	DIV16U_M	AA,BB,CC,ARIT00

; ZOBRAZOVANI CISLA (POTENCIALNE 16BITOVE)
; - NASTAVENO TAK, ABY BYLO ZOBRAZENO JEN 8BITOVE CISLO V REGISTRU "W" (3 CISLICE)
; - BUDE SE POUZIVAT DELENI "DIV16U" A VYPIS ZNAKU NA DISPLEJ "PUTZNAK"
ZOBRAZ16	ZOBR16_M	0,0,B'0000111',ZNAK_0,PUTZNAK,DIV16U,AA,BB,CC

	END
